using System;
using System.Collections.Generic;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading;

namespace Chinwag {
	class Chinwag {
		static void Main(string[] args) {
			if (args.Length == 0) {
				Console.WriteLine("You must specify either 'client' or 'server'.");
				return;
			}

			switch (args[0]) {
				case "client":
					ChinwagClient client = new ChinwagClient();
					client.Run();
					break;
				case "server":
					ChinwagServer server = new ChinwagServer();
					server.Listen();
					server.Run();
					break;
			}
		}
	}

	class ChinwagClient {
		Socket ServerConn;
		bool Running = true;
		byte[] SocketBuffer = new byte[1024];
		string OurName = "Anonymous";

		public void SendText(string text) {
			text = string.Format("<{0}> {1}", OurName, text);
			ServerConn.Send(Encoding.ASCII.GetBytes(text));
		}

		public void Run() {
			string input;

			while (Running) {
				input = Console.ReadLine();

				if (input.StartsWith("/")) {
					// this might be a command
					if (ParseCommand(input)) {
						// command was executed successfully
					} else {
						SendText(input);
					}
				} else {
					SendText(input);
				}
			}
		}

		bool ParseCommand(string command) {
			try {
				string[] splitcomm = command.Split(new char[] { ' ' });

				switch (splitcomm[0]) {
					case "/connect":
						DoConnect(splitcomm[1]);
						return true;
					case "/name":
						OurName = splitcomm[1];
						return true;
				}
			} catch {
				return false;
			}

			return false;
		}

		void DoConnect(string ipaddress) {
			IPEndPoint remoteep = new IPEndPoint(IPAddress.Parse(ipaddress), 32768);
			ServerConn = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
			ServerConn.Blocking = false;
			ServerConn.BeginConnect(remoteep, new AsyncCallback(OnClientConnect), ServerConn);
		}

		public void OnClientConnect(IAsyncResult ar) {
			ServerConn = (Socket)ar.AsyncState;

			if (ServerConn.Connected) {
				ServerConn.BeginReceive(SocketBuffer, 0, SocketBuffer.Length, SocketFlags.None, OnSocketReceive, ServerConn);
			}
		}

		void OnSocketReceive(IAsyncResult ar) {
			int bytes = ServerConn.EndReceive(ar);
			ServerConn.BeginReceive(SocketBuffer, 0, SocketBuffer.Length, SocketFlags.None, OnSocketReceive, ServerConn);

			string text = Encoding.ASCII.GetString(SocketBuffer, 0, bytes);
			Console.WriteLine(text);
		}
	}

	class ChinwagServer {
		Socket OurServerConn;
		List<Socket> RemoteConns = new List<Socket>();

		byte[] SocketBuffer = new byte[1024];

		public void Run() {
			while (true) {
				Thread.Sleep(Timeout.Infinite);
			}
		}

		public void Listen() {
			IPEndPoint local_ep = new IPEndPoint(IPAddress.Any, 32768);

			OurServerConn = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
			OurServerConn.Bind(local_ep);
			OurServerConn.Listen(10);

			OurServerConn.BeginAccept(new AsyncCallback(OnIncomingConnection), OurServerConn);
		}

		void OnIncomingConnection(IAsyncResult ar) {
			Socket client = ((Socket)ar.AsyncState).EndAccept(ar);
			RemoteConns.Add(client);
			client.BeginReceive(SocketBuffer, 0, SocketBuffer.Length, SocketFlags.None, OnSocketReceive, client);
			OurServerConn.BeginAccept(new AsyncCallback(OnIncomingConnection), OurServerConn);

			Console.WriteLine("A new client has connected.");
		}

		void OnSocketReceive(IAsyncResult ar) {
			Socket client = (Socket)ar.AsyncState;
			int bytes = client.EndReceive(ar);
			client.BeginReceive(SocketBuffer, 0, SocketBuffer.Length, SocketFlags.None, OnSocketReceive, client);

			string text = Encoding.ASCII.GetString(SocketBuffer, 0, bytes);
			Console.WriteLine(text);
			SendToAll(client, text);
		}

		void SendToAll(Socket from, string text) {
			foreach (Socket sock in RemoteConns) {
				if (sock == from) continue;

				sock.Send(Encoding.ASCII.GetBytes(text));
			}
		}
	}
}